/*
 *  Copyright (C) 2004 Christophe Fergeau
 *
 * 
 *  The code contained in this file is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either version
 *  2.1 of the License, or (at your option) any later version.
 *  
 *  This file is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this code; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>
#include <glib.h>
#include <glib-object.h>
#include <glib/gi18n.h>
#include <libbonobo.h>
#include <libitunesdb/ipod-detect.h>

/*#include "ipod-calendar-sync.h"*/
#include "ipod-contact-sync.h"

static gchar *
get_ipod_path (void)
{
	gchar *res;
	GList *ipods;

	ipods = ipod_enumerate ();
	if (ipods == NULL) {
		return NULL;
	}

	res = g_strdup (ipods->data);
	g_list_foreach (ipods, (GFunc)g_free, NULL);
	g_list_free (ipods);

	return res;
}

int
main (int argc, char **argv)
{
	char *dest;
	int result;
	gchar *ipod_path;

	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
        bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
        textdomain (GETTEXT_PACKAGE);

	bonobo_init (&argc, argv);

	ipod_path = get_ipod_path ();
	if (ipod_path == NULL) {
		g_print ("Couldn't find a mounted iPod\n");
		return -1;
	}

	result = eis_sync_contacts_to_ipod (ipod_path);
	g_free (ipod_path);
	if (result != 0) {
		g_print (_("Error exporting contacts to your iPod\n"));
		return -1;
	} else {
		g_print (_("Contacts successfully exported to your iPod\n"));
		return 0;
	}

/*	result = eis_sync_events_to_ipod (dest);*/

	return 0;
}
