/*
 *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
 *  Copyright (C) 2004, Christophe Fergeau <teuf@gnome.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gst/gst.h>

typedef struct _Md5Generator Md5Generator;

struct _Md5Generator {
	gboolean eos;
	gboolean error;
	GstElement *pipeline;
	GstElement *sink;
	gchar *md5;
};

static void
md5_generator_gst_eos_cb (GstElement *element, Md5Generator *md)
{
	if (md->eos) {
		g_warning ("RENTERED EOS!");
		return;
	}
	gst_element_set_state (md->pipeline, GST_STATE_NULL);
	g_object_get (G_OBJECT (md->sink), "md5", &md->md5, NULL);

	md->eos = TRUE;
}


static void
md5_generator_gst_error_cb (GstElement *element,
			    GstElement *source,
			    GError *error,
			    gchar *debug,
			    Md5Generator *md)
{
	g_warning ("caught error");
	md->error = TRUE;
}


gchar *
md5_generator_get_md5 (const char *uri)
{
	GstElement *pipeline = NULL;
	GstElement *gnomevfssrc = NULL;
/*	GstElement *typefind = NULL;*/
	GstElement *spider = NULL;
	GstElement *sink;
	GstCaps *filtercaps = NULL;
	gchar *result;
	Md5Generator *md;

	if (uri == NULL) {
		return NULL;
	}

 	md = g_new0 (Md5Generator, 1);
	if (md == NULL) {
		return NULL;
	}

	/* The main tagfinding pipeline looks like this:
	 * gnomevfssrc ! typefind ! spider ! application/x-gst-tags ! fakesink
	 */
	pipeline = gst_pipeline_new ("pipeline");

	g_signal_connect (pipeline, "error", G_CALLBACK (md5_generator_gst_error_cb), md);

	if (!(gnomevfssrc = gst_element_factory_make ("gnomevfssrc", "gnomevfssrc")))
		goto missing_plugin;
	gst_bin_add (GST_BIN (pipeline), gnomevfssrc);		  
	g_object_set (G_OBJECT (gnomevfssrc), "location", uri, NULL);
/*	plugin_name = "typefind";
	if (!(typefind = gst_element_factory_make (plugin_name, plugin_name)))
		goto missing_plugin;
	g_signal_connect_object (typefind, "have_type", G_CALLBACK (md5_generator_gst_typefind_cb), md, 0);
	gst_bin_add (GST_BIN (pipeline), typefind);		  */
	if (!(spider = gst_element_factory_make ("spider", "spider")))
 		goto missing_plugin;
	gst_bin_add (GST_BIN (pipeline), spider);		  
	if (!(md->sink = gst_element_factory_make ("md5sink", "md5sink")))
		goto missing_plugin;
	gst_bin_add (GST_BIN (pipeline), md->sink);		  
	g_signal_connect (md->sink, "eos", G_CALLBACK (md5_generator_gst_eos_cb), md);

	gst_element_link_many (gnomevfssrc, /*typefind,*/ spider, NULL);

	filtercaps = gst_caps_new_simple ("audio/mpeg", NULL);
	gst_element_link_filtered (spider, md->sink, filtercaps);
	gst_caps_free (filtercaps);

	md->pipeline = pipeline;
	gst_element_set_state (pipeline, GST_STATE_PLAYING);
	while (gst_bin_iterate (GST_BIN (pipeline))
	       && !md->error
	       && !md->eos)
		;

	
	gst_element_set_state (pipeline, GST_STATE_NULL);

	if (!md->error) {
		goto out;
	}

 missing_plugin:
	g_free (md->md5); 
	md->md5 = NULL;
 out:
	if (pipeline != NULL)
		gst_object_unref (GST_OBJECT (pipeline));
	md->pipeline = NULL;
	result = md->md5;	
	g_free (md);

	return result;
}
