/*
 *  Copyright (C) 2004 Christophe Fergeau
 *
 * 
 *  The code contained in this file is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either version
 *  2.1 of the License, or (at your option) any later version.
 *  
 *  This file is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this code; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <config.h>

#include <libebook/e-book.h>
#include "ipod-contact-sync.h"
#include "ipod-utils.h"

static gchar *
source_to_vcard_string (ESource *esource)
{
	EBook *ebook;
	EBookQuery *query;
	GList *contacts = NULL;
	GList *c;
	GString *vcf;
	gchar *result;

	ebook = e_book_new (esource, NULL);
	if (ebook == NULL) {
		return NULL;
	}

	e_book_open (ebook, TRUE, NULL);

	query = e_book_query_field_exists (E_CONTACT_FULL_NAME);
	
	if (e_book_get_contacts (ebook, query, &contacts, NULL) == FALSE) {
		e_book_query_unref (query);
		g_object_unref (ebook);
		return NULL;
	}
	e_book_query_unref (query);

	vcf = g_string_new (NULL);
	for (c = contacts; c != NULL; c = c->next) {
		EContact *contact;
		gchar *tmp;

		contact = E_CONTACT (c->data);
		tmp = e_vcard_to_string (E_VCARD (contact), 
					 EVC_FORMAT_VCARD_30);
		g_string_append (vcf, tmp);
		g_string_append (vcf, "\r\n");
		g_free (tmp);
		g_object_unref (contact);
	}
	g_list_free (contacts);
	g_object_unref (ebook);
	result = vcf->str;
	g_string_free (vcf, FALSE);
	
	return result;
}


int
eis_sync_contacts_to_ipod (const gchar *mount_point)
{
	GSList *it;
	GString *vcf;
	int res = 0;
	gunichar2 *vcf_utf16;
	GSList *sources;

	vcf = g_string_new (NULL);

	sources = eis_get_sources (EVO_GCONF_EBOOK_SOURCES);

	for (it = sources; it != NULL; it = it->next) {
		gchar *data;
		data = source_to_vcard_string (E_SOURCE (it->data));
		if ((data != NULL) && (*data != '\0')) {
			g_string_append (vcf, data);
		}
		g_free (data);
	}

	if (vcf->str == NULL) {
		/* FIXME: return a sensible value */
		g_string_free (vcf, TRUE);
		return -1;
	}	

	g_assert (g_utf8_validate (vcf->str, -1, NULL));

	/* FIXME: the resulting string will contain fields saying they are 
	 * encoded in UTF-8 while they actually have UTF-16 content. The iPod
	 * doesn't seem to care though.
	 */

	res = ipod_write_contacts (mount_point, "evolution.vcf", vcf->str);
	g_string_free (vcf, TRUE);

	g_free (vcf_utf16);

	/*	if (res != 0) {
		g_print ("Error writing to iPod: %u\n", res);
		}*/
	
	return res;
}
