/*
 *  Copyright (C) 2003, 2004 Colin Walters <walters@verbum.org>
 *  Copyright (C) 2004 Christophe Fergeau  
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <glib.h>
#include <libgnomevfs/gnome-vfs-utils.h>
#include <libitunesdb/ipoddb.h>
#include <libxml/entities.h>
#include <libxml/SAX.h>
#include <libxml/parserInternals.h>
#include <string.h>

#include "rhythmdb-playlist-parse.h"

static iPodSong *
get_ipod_song_from_uri (iPodDB *db, const char *uri)
{
	iPodSong *song;
	iPodSongIterator *it;

	g_assert (uri != NULL);

	it = ipod_song_iterator_new (db);
	if (it == NULL) {
		printf ("Couldn't create song iterator\n");
		return NULL;
	}

	for (song = ipod_song_iterator_next (it); song != NULL; song = ipod_song_iterator_next (it)) {
		if (song->pc_path_locale == NULL) {
			continue;
		}

		if (!strcmp (song->pc_path_locale, uri)) {
			break;
		}
	}

	g_free (it);

	return song;
}



static void
load_playlist_from_xml (iPodDB *db, xmlNodePtr node)
{
	xmlNodePtr child;
	char *tmp;
	iPodPlaylist *playlist;

	tmp = xmlGetProp (node, "type");
	if ((tmp == NULL) || (!strcmp (tmp, "automatic"))) {
		g_free (tmp);
		return;
	}
	g_free (tmp);

	tmp = xmlGetProp (node, "name");
	if (tmp == NULL) {
		return;
	}
	playlist = ipod_playlist_new (tmp);
	g_free (tmp);
	ipod_db_add_playlist (db, playlist);


	for (child = node->children; child; child = child->next) {	  
		if (xmlNodeIsText (child))
			continue;
		
		if (!strcmp (child->name, "location")) {
			iPodSong *song;
			char *location;

			location = xmlNodeGetContent (child);
 
			song = get_ipod_song_from_uri (db, location);
			g_free (location);
			if (song != NULL) {
				ipod_db_add_song_to_playlist (db, playlist, 
							      song);
			}
		}
       }
}




void
rb_load_playlists (iPodDB *db, const char *file)
{  
	xmlDocPtr doc;
	xmlNodePtr root;
	xmlNodePtr child;
	
	if (!g_file_test (file, G_FILE_TEST_EXISTS)) {
		return;
	}

	doc = xmlParseFile (file);
	
	if (doc == NULL) {
		return;
	}

	root = xmlDocGetRootElement (doc);
	
	for (child = root->children; child; child = child->next) {
		if (xmlNodeIsText (child))
			continue;

		 load_playlist_from_xml (db, child);
	}

	xmlFreeDoc (doc);
}
